import requests
import urllib

from django.conf import settings
from django.contrib.auth import get_user_model, login
from django.contrib import messages
from django.http.response import HttpResponseRedirect
from django.shortcuts import redirect
from django.utils.module_loading import import_string

from django_salesforce_oauth.oauth import OAuth
from django_salesforce_oauth.utils import get_or_create_user

CALLBACK_ERROR_MESSAGE = "CUSTOM_CALLBACK must return a user object or a redirect"


def oauth(request, domain="login"):
    """
    View for initiating OAuth with Salesforce
    """
    url = f"https://{domain}.salesforce.com/services/oauth2/authorize"

    url_args = {
        "client_id": settings.SFDC_CONSUMER_KEY,
        "response_type": "code",
        "redirect_uri": settings.OAUTH_REDIRECT_URI,
        "scope": settings.SCOPES,
        "state": domain,
    }
    args = urllib.parse.urlencode(url_args)

    url = f"{url}?{args}"

    return redirect(url)


def oauth_callback(request):
    """
    View behind the callback URI provided to Salesforce
    """
    code = request.GET.get("code")
    state = request.GET.get("state")

    url = f"https://{state}.salesforce.com/services/oauth2/token"

    if not code:
        messages.error(request, "Unable to authenticate with Salesforce")
        return redirect("index")

    data = {
        "client_id": settings.SFDC_CONSUMER_KEY,
        "client_secret": settings.SFDC_CONSUMER_SECRET,
        "redirect_uri": settings.OAUTH_REDIRECT_URI,
        "grant_type": "authorization_code",
        "code": code,
    }
    response = requests.post(url, data)

    oauth = OAuth(response.json())

    if hasattr(settings, "CUSTOM_CALLBACK"):
        custom_callback = import_string(settings.CUSTOM_CALLBACK)
        response = custom_callback(request, oauth)
        is_user = type(response) == get_user_model()
        is_redirect = type(response) == HttpResponseRedirect
        assert is_user or is_redirect, CALLBACK_ERROR_MESSAGE
        if is_redirect:
            return response
        else:
            user = response
    else:
        user = get_or_create_user(oauth)

    login(request, user)

    messages.info(request, "Authentication with Salesforce successful!")

    return redirect(settings.LOGIN_REDIRECT_URL)



import requests
from django.utils.crypto import get_random_string


class OAuth:
    def __init__(self, token_data):
        self.token_data = token_data
        self._salesforce_user = None
        self._password = get_random_string(length=16)

    def _get_salesforce_user(self):
        sf_id_url = self.token_data.get("id")
        access_token = self.token_data.get("access_token")
        response = requests.get(
            sf_id_url, headers={"Authorization": f"Bearer {access_token}"}
        )

        assert (
            response.status_code == 200
        ), f"Could not retrieve user from Salesforce: {response.reason}"
        return response.json()

    @property
    def salesforce_user(self):
        if not self._salesforce_user:
            self._salesforce_user = self._get_salesforce_user()
        return self._salesforce_user

    @property
    def id(self):
        return self.salesforce_user.get("user_id")

    @property
    def first_name(self):
        return self.salesforce_user.get("first_name")

    @property
    def last_name(self):
        return self.salesforce_user.get("last_name")

    @property
    def username(self):
        return self.salesforce_user.get("username")

    @property
    def email(self):
        return self.salesforce_user.get("email")

    @property
    def organization_id(self):
        return self.salesforce_user.get("organization_id")

    @property
    def instance_url(self):
        return self.token_data.get("instance_url")

    @property
    def access_token(self):
        return self.token_data.get("access_token")

    @property
    def refresh_token(self):
        return self.token_data.get("refresh_token")

    @property
    def issued_at(self):
        return self.token_data.get("issued_at")

    @property
    def password(self):
        """
        Not intended to be used other than to feed to the Django user table
        """
        return self._password



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['django_salesforce_oauth']

package_data = \
{'': ['*']}

install_requires = \
['django>=3.0.7,<4.0.0', 'requests>=2.24.0,<3.0.0']

setup_kwargs = {
    'name': 'django-salesforce-oauth',
    'version': '0.3.3',
    'description': 'Simple package for creating and signing users into your Django site using Salesforce as an OAuth provider',
    'long_description': '# Quick start\n\nAssuming you\'ve already configured an app in your Salesforce instance to serve\nas an OAuth provider, the following should get you up and running.\n\n## Install\n\n`pip install django-salesforce-oauth`\n\n## Settings\n\nAdd the app to your `INSTALLED_APPS` in your django settings (`settings.py`):\n\n```python\nINSTALLED_APPS = [\n    # ...\n    "django_salesforce_oauth",\n]\n```\n\nAdd the following required variables to your `settings.py`:\n\n```python\nSCOPES = "YOUR SCOPES"  # space delimited, e.g., "id api refresh_token"\nSFDC_CONSUMER_KEY = "YOUR KEY"\nSFDC_CONSUMER_SECRET = "YOUR SECRET"\nOAUTH_REDIRECT_URI = "{YOUR DOMAIN}/oauth/callback/"\n\n# Optional, but Django provides a default you likely don\'t want\nLOGIN_REDIRECT_URL = "/"\n```\n\n## Urls\n\nAdd `django-salesforce-oauth`\'s urls to your main `urls.py`.\n\n```python\nfrom django.urls import path, include\n\nurlpatterns = [\n    # ...\n    path("oauth/", include("django_salesforce_oauth.urls")),\n]\n```\n\nThen redirect sign-in requests to the `oauth` namespace.\n\n### View example\n\n```python\nfrom django.shortcuts import redirect\n\ndef your_view(request):\n    return redirect("oauth")  # or "oauth-sandbox"\n```\n\n### Template example\n\n```html\n<a href="{% url \'oauth\' %}" class="btn btn-primary">Login</a>\n```\n\n# Advanced usage\n\n## Custom callback\n\nYou likely will want to customize what happens after the OAuth flow is complete instead of simply\ngetting or creating a user. This can be done by specifying the following in your `settings.py`.\n\n```python\nCUSTOM_CALLBACK = "path.to.module.your_callback_function"\n```\n\n`your_callback_function` must accept the following two arguments:\n\n1. the request object (useful in case you want to handle redirection yourself)\n2. the OAuth object (contains all token and user data)\n\nIf you do not return a redirect from `your_callback_function`, it\'s expected it will return\na user object. In this case the user will then be signed in and redirected to\n`settings.LOGIN_REDIRECT_URL` (which you\'ll most likely want to set in your `settings.py`).\n\n### Customizing the callback URI\n\nBy default the view behind the `oauth-callback` namespace, specified in the `django_salesforce_oauth`\'s app\'s `urls.py`, is what needs to match `settings.OAUTH_REDIRECT_URI`.\nBut this can be customized by pointing it to some other url and registering the view wherever\nyou\'d like it declared.\n\n```python\n# urls.py\n\nfrom django_salesforce_oauth.views import oauth_callback\n\nurlpatterns = [\n    # ...\n    # pass {"domain": "test"} to use a sandbox\n    path("my/custom/url", oauth_callback, {"domain": "login"}, name="custom-oauth-callback"),\n]\n```\n\n# Example project\n\nThe example project provides a full example of how to use this package,\nbut since it\'s an integration, there\'s a few steps to actually running it.\n\n## SFDC\n\nConfigure a SFDC OAuth app with which you can OAuth against.\n\n## .env\n\nPlace a `.env` file inside the `project` folder that contains the following keys\nfrom the OAuth app you configured above:\n\n```\nSFDC_CONSUMER_KEY=some_key\nSFDC_CONSUMER_SECRET=secret_stuff\n```\n\n## django\n\nrun migrations and start the server!\n\n---\n\nThis project uses [poetry](https://python-poetry.org/) for dependency management\nand packaging.\n',
    'author': 'Alex Drozd',
    'author_email': 'drozdster@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/brno32/django-salesforce-oauth',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



from django.contrib.auth import get_user_model

from django_salesforce_oauth.oauth import OAuth


def get_or_create_user(oauth: OAuth):
    """
    Accepts an OAuth object and retrieves a User, creating one if it doesn't exist
    """
    salesforce_id = oauth.id
    email = oauth.email
    password = oauth.password

    User = get_user_model()

    user = User.objects.filter(username=salesforce_id).first()

    if not user:
        user = User.objects.create_user(salesforce_id, email, password)

    return user



from django.contrib.auth.views import LogoutView
from django.urls import path

from django_salesforce_oauth.views import oauth, oauth_callback

urlpatterns = [
    path("", oauth, {"domain": "login"}, name="oauth"),
    path("sandbox/", oauth, {"domain": "test"}, name="oauth-sandbox"),
    path("callback/", oauth_callback, name="oauth-callback"),
    path("logout/", LogoutView.as_view(), name="logout"),
]
